/**@file

@brief This library contains generic functions for faceplate animation.

First two parameters are dpes and values, which should be transfered straight from callback.
First two places in dpes/values are reserved for frontend statuses.

All animation functions check system validness before animation which reflects on color.
Also they are capable to animate disconnection.

Functions are supposed to have corresponding field(s) in the faceplate named in a special way. Function description contains an information about required fields.
Naming convention for graphical objects:
- for analog values object should have the same name as dpe ("PosSt" for ".ProcessInput.PosSt" dpe);
- for bit values object should have a bit's name in capital that responds to bit's position constant (e.g. "IOERRORW").

For bit animation bit position constant should be used (e.g. "UN_STSREG01_IOERRORW"). 
This constant contains 3 parts joined with underscore:
- first part is reflect package prefix and does not participate in animation;
- second part contains an information about StsReg where a bit is located;
- third part is bit name ($BIT_NAME$ in other examples).

@see cpcGenericAnimation.ctl

@author Alexey Merezhin (EN-ICE-SIC)

@copyright
        &copy;Copyright CERN 2013 - all rights reserved

@par Creation Date
  03.10.2011

@par Modification History

    10/10/2012: Alexey
        refactor function names and update documentation

    08/02/2012: Alexey
        [UCPC-656] fixed collision in cpcFaceplate_fetchAnimationCBOnlineValue (use dot in the beginning of dpe pattern)

    20/01/2012: Alexey
        cpcFaceplate_animateTextParameter, cpcFaceplate_animateTranslatedValue and cpcFaceplate_animateInterlock
        extracted cpcFaceplate_fetchStsRegBitAndValidness from cpcFaceplate_animateStsRegBit

*/

/**Animate text parameter.

Faceplate should contain unFaceplate_DisplayColor.pnl or unFaceplate_DisplayNormal.pnl graphical object with the '$paramName$' name.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param paramName name of parameter to animate
@param value value of parameter to animate
*/
void cpcFaceplate_animateTextParameter(dyn_string dpes, dyn_string values, string paramName, string value) {
    string color = (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value) ? "unDataNotValid" : "unDisplayValue_Parameter";

    if(g_bSystemConnected) {
        setMultiValue(paramName + ".display", "foreCol", color, paramName + ".display", "text", value);
    }
}

/**Fetch value of dpe and animate a translation from dict.

Faceplate should contain unFaceplate_DisplayColor.pnl or unFaceplate_DisplayNormal.pnl graphical object with the '$dpeName$' name.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of dpe to animate
@param dpeUnit dpe unit
@param dpeFormat dpe format
@param dict mapping with translated values
@param color field color on valid state
*/
void cpcFaceplate_animateTranslatedValue(dyn_string dpes, dyn_string values, string dpeName, string dpeUnit, string dpeFormat, mapping dict, string color) {
    int value;
    bool invalid;

    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, dpeName, value, invalid);
    invalid = invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value);
    
    cpcGenericObject_animateTranslatedValue(dpeName, dict, dpeUnit, dpeFormat, value, color, invalid);
}

/**Animate interlock.

Faceplate should contain unFaceplate_ColoredSquare.pnl graphical object with the '$interlockName$' name ('FuStopISt' for example).

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param bitName interlock bit constant in StsReg (e.g. UN_STSREG01_STARTAL); see detailed description for more info
@param interlockName interlock DPE name
@param unAckBitName AlUnAck (alarm un-acknowledge) bit constant as string
*/
void cpcFaceplate_animateInterlock(dyn_string dpes, dyn_string values, string bitName, string interlockName, string unAckBitName = "UN_STSREG01_ALUNACK") {
  if (dynlen(dpes) > 0) { // online animation
    bool value, invalid, unAck, unAckInvalid;

    int interlockState = cpcFaceplate_fetchAnimationCBValue(dpes, values, interlockName);
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, bitName, value, invalid);
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, unAckBitName, unAck, unAckInvalid);

    if (g_bSystemConnected)
        cpcGenericObject_ColorBoxAnimateInterlock(interlockName, interlockState, value, unAck,
                    invalid || unAckInvalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
  } else {
    unGenericObject_ColoredSquareDisconnect(interlockName);
  }
}

/**Animate analog value.

Faceplate should contain unFaceplate_DisplayColor.pnl or unFaceplate_DisplayNormal.pnl graphical object with the '$dpeName$' name ("PosSt" for example).

@todo: should be replaced with cpcFaceplate_animateAnalogValue

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of dpe to animate
@param dpeUnit dpe unit
@param dpeFormat dpe format
@param color field color on valid state
*/
void cpcFaceplate_animateOnlineValue(dyn_string dpes, dyn_string values, string dpeName, string dpeUnit, string dpeFormat, string color) {
  anytype value;
  bool invalid;
  if (dynlen(dpes) > 0) { // online animation
    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, dpeName, value, invalid);
    if (g_bSystemConnected) {
        unGenericObject_DisplayValue(dpeFormat, dpeUnit, value, dpeName, color, 
            invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
    }
  } else {
    unGenericObject_DisplayValueDisconnect(dpeName);
  }
}
/**Animate analog value.

Faceplate should contain unFaceplate_DisplayColor.pnl or unFaceplate_DisplayNormal.pnl graphical object with the '$dpeName$' name ("PosSt" for example).

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of dpe to animate
@param refPrefix prefix for unit and format in g_params
@param color field color on valid state
*/
void cpcFaceplate_animateAnalogValue(dyn_string dpes, dyn_string values, string dpeName, string refPrefix, string color) {
  anytype value;
  bool invalid;
  if (dynlen(dpes) > 0) { // online animation
    string dpeUnit = g_params[refPrefix + "Unit"];
    string dpeFormat = g_params[refPrefix + "Format"];
    cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, dpeName, value, invalid);
    if (g_bSystemConnected) {
        unGenericObject_DisplayValue(dpeFormat, dpeUnit, value, dpeName, color, 
            invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
    }
  } else {
    unGenericObject_DisplayValueDisconnect(dpeName);
  }
}

/**Animate StsReg bit.

Function accept bit's constant name (e.g. "UN_STSREG01_IOERRORW") as bitName. Constant's value should correspond to bit position in StsReg dpe.
Faceplate should contain unFaceplate_Colorbox.pnl graphical object named '$BIT_NAME$' ("IOERRORW" in given example).

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param bitName constant as a string; see detailed description for more info
@param enabled if it's enabled
@param color field color on valid state
*/
void cpcFaceplate_animateStsRegBit(dyn_string dpes, dyn_string values, string bitName, bool enabled, string color) {
  dyn_string parts = strsplit(bitName, "_");
  if (dynlen(dpes) > 0) { // online animation
    bool invalid, value;
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, bitName, value, invalid);

    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate(parts[3], value, enabled, color, 
                    invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
  } else { // disconnection animation
    unGenericObject_ColorBoxDisconnect(makeDynString(parts[3]));
  }
}

/**Animate reverse StsReg bit.

Opposite animation (TRUE->FALSE) of cpcFaceplate_animateStsRegBit.
As bitName should be used normal bit constant.
Faceplate should contain unFaceplate_Colorbox.pnl graphical object named 'NOT_$BIT_NAME$' ("NOT_IOERRORW" in given example).

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@see cpcFaceplate_animateStsRegBit

@param dpes connected dpe names
@param values connected dpe values
@param bitName constant as a string
@param enabled if it's enabled
@param color field color on valid state
*/
void cpcFaceplate_animateStsRegNotBit(dyn_string dpes, dyn_string values, string bitName, bool enabled, string color) {
  dyn_string parts = strsplit(bitName, "_");
  if (dynlen(dpes) > 0) { // online animation
    bool invalid, value;
    cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, bitName, value, invalid);

    if(g_bSystemConnected)
        unGenericObject_ColorBoxAnimate("NOT_" + parts[3], !value, enabled, color, 
                    invalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
  } else { // disconnection animation
    unGenericObject_ColorBoxDisconnect(makeDynString("NOT_" + parts[3]));
  }
}

/**Animate dpe's range.

For given dpe function fetch range properties from dpes/values and animate them accordingly.
Faceplate should contain two unFaceplate_DisplayNormal.pnl graphical objects with the '$DPE_NAME$RangeMax' and '$DPE_NAME$RangeMin' names ("PosStRangeMax" and "PosStRangeMax" for "PosSt" dpe).

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@see cpcFaceplate_animateStsRegBit

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of dpe with range configured to animate
@param rangeType range type, should be statically initialized
@param dpeUnit dpe unit
@param dpeFormat dpe format
*/
void cpcFaceplate_animateRange(dyn_string dpes, dyn_string values, string dpeName, int rangeType, string dpeUnit, string dpeFormat) {
    float rangeMin, rangeMax;
    if (rangeType == DPCONFIG_MINMAX_PVSS_RANGECHECK) {
        rangeMin = cpcFaceplate_fetchAnimationCBValue(dpes, values, dpeName + ":_pv_range.._min");
        rangeMax = cpcFaceplate_fetchAnimationCBValue(dpes, values, dpeName + ":_pv_range.._max");
    } else {
        rangeMin = 0;
        rangeMax = 0;
    }
    if(g_bSystemConnected)
        unGenericObject_DisplayRange(rangeType, dpeFormat, dpeUnit, rangeMin, rangeMax, dpeName + "RangeMin", dpeName + "RangeMax", "unDisplayValue_Parameter");
}

/**Animate 'allow restart needed' bit.

Faceplate should contain unFaceplate_Colorbox.pnl graphical object named "NEEDRESTART".

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
*/
void cpcFaceplate_animateNeedRestart(dyn_string dpes, dyn_string values) {
    bool needRestartInvalid, needRestart;
    if (dynlen(dpes) > 0) { // online animation
        cpcFaceplate_fetchStsRegBitAndValidness(dpes, values, "CPC_STSREG02_NEEDRESTART", needRestart, needRestartInvalid);
        cpcGenericObject_ColorBoxAnimate("NEEDRESTART", g_params["MRestart"] && needRestart == 0, 1, "unFaceplate_WarningActive",
                        needRestartInvalid || (!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
    } else {
        unGenericObject_ColorBoxDisconnect("NEEDRESTART");
    }
}

/**Animate mask event.

Faceplate should contain unFaceplate_Colorbox.pnl graphical object named "eventState".

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
*/
void cpcFaceplate_animateMaskEvent(dyn_string dpes, dyn_string values) {
    bool maskEvent, invalid;
    if (dynlen(dpes) > 0) { // online animation
        cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, "eventMask", maskEvent, invalid);
        if(g_bSystemConnected)
            unGenericObject_ColorBoxAnimate("eventState", !maskEvent, 1, "unFaceplate_WarningActive", 
                        invalid || cpcFaceplate_connectionValid(values));
    } else {
        unGenericObject_ColorBoxDisconnect("eventState");
    }
}

/**Fetch value and validness of given bit.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param bitName constant as a string
@param value output
@param invalid output
*/
void cpcFaceplate_fetchStsRegBitAndValidness(dyn_string dpes, dyn_string values, string bitName, bool &value, bool &invalid) {
    int stsReg;
    dyn_string exceptionInfo;
    int bitPosition = cpcConfigGenericFunctions_getConstantByName(bitName);
    if (bitPosition < 0) {
        fwException_raise(exceptionInfo,"ERROR", "cpcFaceplate_fetchStsRegBitAndValidness: no position for " + bitName, "EMPTY");
        return;
    }
    dyn_string parts = strsplit(bitName, "_");
    if (dynlen(parts) != 3) {
        fwException_raise(exceptionInfo,"ERROR", "cpcFaceplate_fetchStsRegBitAndValidness: wrong bit name " + bitName, "EMPTY");
        return;
    }
    if (parts[2] == "STSREG01") {
        cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, "StsReg01", stsReg, invalid);
    } else if (parts[2] == "STSREG02") {
        cpcFaceplate_fetchAnimationCBOnlineValue(dpes, values, "StsReg02", stsReg, invalid);
    } else {
        fwException_raise(exceptionInfo,"ERROR", "cpcFaceplate_fetchStsRegBitAndValidness: wrong stsreg " + bitName, "EMPTY");
        return;
    }
    value = cpcGenericObject_getBit(stsReg, bitPosition);
}

/**Fetch value and validness of given dpe.
Returns as fast as both value and invalid are found.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of dpe
@param value output
@param invalid output
*/
void cpcFaceplate_fetchAnimationCBOnlineValue(dyn_string dpes, dyn_string values, string dpeName, anytype &value, bool &invalid) {
    string dpeValue = "." + dpeName + ":_online.._value";
    string dpeValid = "." + dpeName + ":_online.._invalid";
    bool valueFound = false, invalidFound = false;
    for (int i = 1; i <= dynlen(dpes); i++) {
        if (strpos(dpes[i], dpeValue) >= 0) {
            value = values[i];
            valueFound = true;
        }
        if (strpos(dpes[i], dpeValid) >= 0) {
            invalid = values[i];
            invalidFound = true;
        }
        if (valueFound && invalidFound) return;
    }
}

/**Fetch value of given dpe.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@param dpes connected dpe names
@param values connected dpe values
@param dpeName name of dpe
@return dpe value or CPC_FACEPLATE_MISSED_VALUE if dpe's not found in dpes/values
*/
anytype cpcFaceplate_fetchAnimationCBValue(dyn_string dpes, dyn_string values, string dpeName) {
    for (int i = 1; i <= dynlen(dpes); i++) {
        if (strpos(dpes[i], dpeName) >= 0) {
            return values[i];
        }
    }
    return CPC_FACEPLATE_MISSED_VALUE;
}

/**Return front-end validness.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  UI

@todo rename me to cpcFaceplate_frontEndValid
  
@return is front-end valid
*/
bool cpcFaceplate_connectionValid(dyn_string values) {
  return dynlen(values) > 0 && ((!values[2]) || (values[1] > c_unSystemIntegrity_no_alarm_value));
}

